@@ -14,25 +14,32 @@ See the License for the specific language governing permissions and
 limitations under the License.
 '''
 
+from collections import defaultdict
+from datetime import datetime
+from fnmatch import fnmatch
+from importlib.abc import Loader
+from typing import Any, DefaultDict, Dict, Iterator, List, Tuple
 import argparse
 import base64
-from datetime import datetime
 import importlib.util
-from importlib.abc import Loader
-from collections import defaultdict
 import json
 import logging
 import os
 import sys
-from typing import Any, DefaultDict, Dict, Iterator, List, Tuple
 import zipfile
-from schema import (Optional, Or, Schema, SchemaError, SchemaMissingKeyError,
+
+from schema import (Optional, SchemaError, SchemaMissingKeyError,
                     SchemaForbiddenKeyError, SchemaUnexpectedTypeError)
+import boto3
 import yaml
 
-import boto3
+from panther_analysis_tool.schemas import TYPE_SCHEMA, GLOBAL_SCHEMA, POLICY_SCHEMA, RULE_SCHEMA
 
-HELPERS_LOCATION = 'global_helpers'
+HELPERS_LOCATION = './global_helpers'
+
+HELPERS_PATH_PATTERN = '*/global_helpers'
+RULES_PATH_PATTERN = '*rules*'
+POLICIES_PATH_PATTERN = '*policies*'
 
 
 class TestCase():
@@ -51,100 +58,6 @@ class TestCase():
         return self._data.get(arg, default)
 
 
-TYPE_SCHEMA = Schema({
-    'AnalysisType': Or("policy", "rule", "global"),
-},
-                     ignore_extra_keys=True)
-
-GLOBAL_SCHEMA = Schema(
-    {
-        'AnalysisType': Or("global"),
-        'Filename': str,
-        'GlobalID': str,
-        Optional('Description'): str,
-        Optional('Tags'): [str],
-    },
-    ignore_extra_keys=False)
-
-POLICY_SCHEMA = Schema(
-    {
-        'AnalysisType':
-            Or("policy"),
-        'Enabled':
-            bool,
-        'Filename':
-            str,
-        'PolicyID':
-            str,
-        'ResourceTypes': [str],
-        'Severity':
-            Or("Info", "Low", "Medium", "High", "Critical"),
-        Optional('ActionDelaySeconds'):
-            int,
-        Optional('AutoRemediationID'):
-            str,
-        Optional('AutoRemediationParameters'):
-            object,
-        Optional('Description'):
-            str,
-        Optional('DisplayName'):
-            str,
-        Optional('Reference'):
-            str,
-        Optional('Runbook'):
-            str,
-        Optional('Suppressions'): [str],
-        Optional('Tags'): [str],
-        Optional('Reports'): {
-            str: object
-        },
-        Optional('Tests'): [{
-            'Name': str,
-            'ResourceType': str,
-            'ExpectedResult': bool,
-            'Resource': object,
-        }],
-    },
-    ignore_extra_keys=False)
-
-RULE_SCHEMA = Schema(
-    {
-        'AnalysisType':
-            Or("rule"),
-        'Enabled':
-            bool,
-        'Filename':
-            str,
-        'RuleID':
-            str,
-        'LogTypes': [str],
-        'Severity':
-            Or("Info", "Low", "Medium", "High", "Critical"),
-        Optional('Description'):
-            str,
-        Optional('DedupPeriodMinutes'):
-            int,
-        Optional('DisplayName'):
-            str,
-        Optional('Reference'):
-            str,
-        Optional('Runbook'):
-            str,
-        Optional('Suppressions'): [str],
-        Optional('Tags'): [str],
-        Optional('Reports'): {
-            str: object
-        },
-        Optional('Tests'): [{
-            'Name': str,
-            'LogType': str,
-            'ExpectedResult': bool,
-            'Log': object,
-        }],
-    },
-    ignore_extra_keys=False)
-
-
 def load_module(filename: str) -> Tuple[Any, Any]:
     """Loads the analysis function module from a file.
 
@@ -179,15 +92,27 @@ def load_analysis_specs(directory: str) -> Iterator[Tuple[str, str, Any]]:
     Yields:
         A tuple of the relative filepath, directory name, and loaded analysis specification dict.
     """
-    for dir_name, _, file_list in os.walk(directory):
+    for relative_path, _, file_list in os.walk(directory):
+        # If the user runs with no path/out args, filter to make sure
+        # we only run folders with valid analysis files.
+        if directory == '.':
+            if not any([
+                    fnmatch(relative_path, path_pattern)
+                    for path_pattern in (HELPERS_PATH_PATTERN,
+                                         RULES_PATH_PATTERN,
+                                         POLICIES_PATH_PATTERN)
+            ]):
+                logging.debug('Skipping path %s', relative_path)
+                continue
         for filename in sorted(file_list):
-            spec_filename = os.path.join(dir_name, filename)
-            if filename.endswith('.yaml') or filename.endswith('.yml'):
+            spec_filename = os.path.join(relative_path, filename)
+            if fnmatch(filename, '*.y*ml'):
                 with open(spec_filename, 'r') as spec_file_obj:
-                    yield spec_filename, dir_name, yaml.safe_load(spec_file_obj)
-            if filename.endswith('.json'):
+                    yield spec_filename, relative_path, yaml.safe_load(
+                        spec_file_obj)
+            if fnmatch(filename, '*.json'):
                 with open(spec_filename, 'r') as spec_file_obj:
-                    yield spec_filename, dir_name, json.load(spec_file_obj)
+                    yield spec_filename, relative_path, json.load(spec_file_obj)
 
 
 def datetime_converted(obj: Any) -> Any:
@@ -227,8 +152,15 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
         ':', '-')
     filename = 'panther-analysis-{}.zip'.format(current_time)
     with zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED) as zip_out:
-        analysis = filter_analysis(list(load_analysis_specs(args.path)),
-                                   args.filter)
+        # Always zip the helpers
+        analysis = []
+        files: Dict[str, Any] = {}
+        for (file_name, f_path, spec) in list(load_analysis_specs(
+                args.path)) + list(load_analysis_specs(HELPERS_LOCATION)):
+            if file_name not in files:
+                analysis.append((file_name, f_path, spec))
+                files[file_name] = None
+        analysis = filter_analysis(analysis, args.filter)
         for analysis_spec_filename, dir_name, analysis_spec in analysis:
             zip_out.write(analysis_spec_filename)
             zip_out.write(os.path.join(dir_name, analysis_spec['Filename']))
@@ -368,22 +300,47 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
         failed_tests = run_tests(analysis_spec, analysis_funcs, failed_tests)
         print('')
 
-    for analysis_id in failed_tests:
-        print("Failed: {}\n\t{}\n".format(analysis_id,
-                                          failed_tests[analysis_id]))
-
-    for spec_filename, spec_error in invalid_specs:
-        print("Invalid: {}\n\t{}\n".format(spec_filename, spec_error))
-
+    print_summary(args.path, len(analysis), failed_tests, invalid_specs)
     return int(bool(failed_tests or invalid_specs)), invalid_specs
 
 
+def print_summary(test_path: str, num_tests: int, failed_tests: List[Any],
+                  invalid_specs: List[Any]) -> None:
+    '''Print a summary of passed, failed, and invalid specs'''
+    print('--------------------------')
+    print('Panther CLI Test Summary')
+    print('\tPath: {}'.format(test_path))
+    print("\tPassed: {}".format(num_tests -
+                                (len(failed_tests) + len(invalid_specs))))
+    print("\tFailed: {}".format(len(failed_tests)))
+    print("\tInvalid: {}\n".format(len(invalid_specs)))
+
+    err_message = "\t{}\n\t\t{}\n"
+
+    if failed_tests:
+        print('--------------------------')
+        print('Failed Tests Summary')
+        for analysis_id in failed_tests:
+            print(err_message.format(analysis_id, failed_tests[analysis_id]))
+
+    if invalid_specs:
+        print('--------------------------')
+        print('Invalid Tests Summary')
+        for spec_filename, spec_error in invalid_specs:
+            print(err_message.format(spec_filename, spec_error))
+
+
 def filter_analysis(analysis: List[Any], filters: Dict[str, List]) -> List[Any]:
     if filters is None:
         return analysis
 
     filtered_analysis = []
     for file_name, dir_name, analysis_spec in analysis:
+        if fnmatch(dir_name, HELPERS_PATH_PATTERN):
+            logging.debug('auto-adding helpers file %s',
+                          os.path.join(file_name))
+            filtered_analysis.append((file_name, dir_name, analysis_spec))
+            continue
         match = True
         for key, values in filters.items():
             spec_value = analysis_spec.get(key, "")
@@ -477,7 +434,7 @@ def setup_parser() -> argparse.ArgumentParser:
         prog='panther_analysis_tool')
     parser.add_argument('--version',
                         action='version',
-                        version='panther_analysis_tool 0.3.1')
+                        version='panther_analysis_tool 0.3.2')
     subparsers = parser.add_subparsers()
 
     test_parser = subparsers.add_parser(
@@ -485,13 +442,15 @@ def setup_parser() -> argparse.ArgumentParser:
         help='Validate analysis specifications and run policy and rule tests.')
     test_parser.add_argument(
         '--path',
+        default='.',
         type=str,
         help='The relative path to Panther policies and rules.',
-        required=True)
+        required=False)
     test_parser.add_argument('--filter',
                              required=False,
                              metavar="KEY=VALUE",
                              nargs='+')
+    test_parser.add_argument('--debug', action='store_true', dest='debug')
     test_parser.set_defaults(func=test_analysis)
 
     zip_parser = subparsers.add_parser(
@@ -501,18 +460,21 @@ def setup_parser() -> argparse.ArgumentParser:
     )
     zip_parser.add_argument(
         '--path',
+        default='.',
         type=str,
         help='The relative path to Panther policies and rules.',
-        required=True)
+        required=False)
     zip_parser.add_argument(
         '--out',
+        default='.',
         type=str,
         help='The path to write zipped policies and rules to.',
-        required=True)
+        required=False)
     zip_parser.add_argument('--filter',
                             required=False,
                             metavar="KEY=VALUE",
                             nargs='+')
+    zip_parser.add_argument('--debug', action='store_true', dest='debug')
     zip_parser.set_defaults(func=zip_analysis)
 
     upload_parser = subparsers.add_parser(
@@ -520,9 +482,10 @@ def setup_parser() -> argparse.ArgumentParser:
         help='Upload specified policies and rules to a Panther deployment.')
     upload_parser.add_argument(
         '--path',
+        default='.',
         type=str,
         help='The relative path to Panther policies and rules.',
-        required=True)
+        required=False)
     upload_parser.add_argument(
         '--out',
         default='.',
@@ -534,6 +497,7 @@ def setup_parser() -> argparse.ArgumentParser:
                                required=False,
                                metavar="KEY=VALUE",
                                nargs='+')
+    upload_parser.add_argument('--debug', action='store_true', dest='debug')
     upload_parser.set_defaults(func=upload_analysis)
 
     return parser
@@ -563,11 +527,12 @@ def parse_filter(filters: List[str]) -> Dict[str, Any]:
 
 
 def run() -> None:
-    logging.basicConfig(format='[%(levelname)s]: %(message)s',
-                        level=logging.INFO)
-
     parser = setup_parser()
     args = parser.parse_args()
+
+    logging.basicConfig(format='[%(levelname)s]: %(message)s',
+                        level=logging.DEBUG if args.debug else logging.INFO)
+
     try:
         if args.filter is not None:
             args.filter = parse_filter(args.filter)
@@ -577,7 +542,7 @@ def run() -> None:
         sys.exit(1)
     except Exception as err:  # pylint: disable=broad-except
         # Catch arbitrary exceptions without printing help message
-        logging.warning('Unexpected exception: "%s"', err)
+        logging.warning('Unhandled exception: "%s"', err)
         sys.exit(1)
 
     if return_code == 1:
